#-------------------------------------------------------------------------------
# Filname:      calcmedian.py
# Description:   This script reads data in "bins" (groups or
#       ranges) and calculates a median. Sample data is
#       household income from the US Census
#       American Community Survey.
#
# Sample file inc.csv: contains income group data for
#       households in Androscoggin County, ME from the
#       2020 American Community Survey 5-year estimates
#       See table B19013 for median income comparison/check
#
#       First column contains the starting
#       number for the range (i.e. less than $10,000 is 0;
#       10k to 14999 is 10000; 15k-19999 is 15000, etc)
#       Second column is the total households for that range
#
# Running the file will ask for the csv filename - easiest if you have it
# stored in the same location as the script - but you can enter the path
# along with the filename and it should work as well.
#
# All the print statements are not needed, just in there to double check
# the calculation,
#
# Author:       Jami Dennis
# Created:      1/10/2019 - updated to Python 3 on 3/22/2022
#-------------------------------------------------------------------------------


import csv

# Function to read in the CSV containing bin data

def read(filename):
    # initialize the directory
    bins = {}
    with open(filename, 'U') as csvfile:
        reader = csv.reader(csvfile)
        next (csvfile) #skip first row headers
        # read in each row into the dictionary
        # make sure to set them as integers
        for row in reader:
            bins[int(row[0])] = int(row[1])

            # print(row) # this just prints your inputs - used for testing

    return bins

# have the user enter the filename
bins = read(input('Enter the filename: '))

# intialize a running total to aggregate the values
total = 0

# determin the half way point simply by divding the total
# population by 2
median_index = (sum(bins.values()))/2

keyList = sorted(bins.keys())

# initialize a dictionary for the cumulative values for each bin
aggList = {0:0}

# step through the sorted dictionary until the cumulative value
# exceeds the mid point (median_index)
for i,value in enumerate(keyList):

    # calculate the cumulative value
    total += bins[value]

    # add cumulative value to aggList
    aggList[i]= total

    # check if the cumulative value has exceeded median_index
    if total > median_index:

        # get the cumulative pop for the bin just before
        # we exceeded the median_index using aggList[i-1]
        # and subtract that from the median_index to find out
        # how much much into the bin it is to reach the
        # median_index
        howMany = median_index - aggList[i-1]
        print(('the halfway point is: ', "{:.2f}".format(median_index)))
        print(('Cumulative pop where halfway point comes after: ', "{:.2f}".format(aggList[i-1])))
        print(('howMany: ', "{:.2f}".format(howMany)))

        # get the population in the bin (Bin) for
        # the keyList[i] bin (the one before we exceeded
        # the median_index
        Bin= bins[keyList[i]]
        print(('The beginning of the Bin the halfway falls into: ', "{:.2f}".format(keyList[i])))
        print(('The beginning of the next bin: ', "{:.2f}".format(keyList[i+1])))
        print(('The span of the Bin: ', "{:.2f}".format((keyList[i+1]-keyList[i]))))
        print(('How many into the bin: ', "{:.2f}".format(Bin)))
        # determine the percent of how far into this bin
        # the median_index falls
        pctInto = howMany / Bin
        print(('Pct into the bin: ', "{:.2f}".format(pctInto)))
        # determine the span of the bin using
        # keyList[i+1]-keyList[i]
        # and multiply it by the percent into the bin
        multiSpan = (keyList[i+1]-keyList[i]) * pctInto
        print(('How many into the Bin it goes: ', "{:.2f}".format(multiSpan)))

        # cacluate the median by adding the result to
        # the number for the beginning of the range
        median = keyList[i] + multiSpan


        print(('the median is: ', "{:.2f}".format(median)))

        # we found the median so break out of the for loop
        break

# mission accomplished